<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:math="http://www.w3.org/2005/xpath-functions/math"
    exclude-result-prefixes="xs math"
    xmlns:ai="http://www.oxygenxml.com/ai/function"
    version="3.0">
    <xsl:template match="topic">
        <xsl:variable name="topicContent" select="."/>
        
        <!-- Step 1 -->
        <xsl:variable name="threeVariantsInstruction" select="'Provide 3 variants of summarizing the given DITA topic,
            that could be later used as short descriptions for this topic. Respond with just the text content, no XML.'"/>
        <xsl:variable name="theThreeVariants" select="ai:transform-content($threeVariantsInstruction, $topicContent)"/>
        
        <!-- Step 2 -->
        <xsl:variable name="advAndDisadvInstr" select="'For each of the given text variants,
            analyze how good a short description it is by enumerating 3 strong points and 3 weak points.'"/>
        <xsl:variable name="assistantAdvAndDisadv" select="ai:transform-content($advAndDisadvInstr, $theThreeVariants)"/>
        
        <!-- Step 3 -->
        <xsl:variable name="mainInstruction" select="'You will act as a technical writer. You are tasked to choose the best short description
            for a topic. You will take into account 3 variants and their strong points and weak points. You should answer with the text
            of the best short description among the 3 variants. Do not generate another variant, but choose one of the 3.'"/>
        <xsl:variable name="assistantBestVariant" select="ai:transform-content(
            $mainInstruction,
            
            $threeVariantsInstruction,
            $theThreeVariants,
            
            $advAndDisadvInstr, 
            $assistantAdvAndDisadv,
            
            $topicContent)"/>
        
        <xsl:text>USER
</xsl:text>
        <xsl:value-of select="$threeVariantsInstruction"/>
        <xsl:text>
            --------------------
        </xsl:text>
        
        <xsl:text>ASSISTANT
</xsl:text>
        <xsl:value-of select="$theThreeVariants"/>
        <xsl:text>
            --------------------
        </xsl:text>
        
        <xsl:text>USER
</xsl:text>
        <xsl:value-of select="$advAndDisadvInstr"/>
        <xsl:text>
            --------------------
        </xsl:text>
        
        <xsl:text>ASSISTANT
</xsl:text>
        <xsl:value-of select="$assistantAdvAndDisadv"/>
        <xsl:text>
            --------------------
        </xsl:text>
        
        <xsl:text>USER
</xsl:text>
        <xsl:value-of select="$mainInstruction"/>
        <xsl:text>
            --------------------
        </xsl:text>
        
        <xsl:text>ASSISTANT
</xsl:text>
        <xsl:value-of select="$assistantBestVariant"/>
        <xsl:text>
            --------------------
        </xsl:text>
    </xsl:template>
</xsl:stylesheet>