<?xml version="1.0" encoding="UTF-8"?>
<sch:schema xmlns:sch="http://purl.oclc.org/dsdl/schematron"
    xmlns:sqf="http://www.schematron-quickfix.com/validator/process" queryBinding="xslt2"
    xmlns="http://www.w3.org/1999/xhtml">
    <sch:ns uri="http://www.oxygenxml.com/ai/function" prefix="ai"/>
    
    <!-- Missing short description -->
    <sch:pattern>
        <sch:rule context="topic">
            <sch:report 
                test="not(shortdesc)"
                role="warn"
                sqf:fix="generate_shortdesc">The short description is missing...
            </sch:report>
            <sqf:fix id="generate_shortdesc">
                <sqf:description><sqf:title>Generate the best short description</sqf:title></sqf:description>
                
                <!-- Step 1: Ask for 3 variants -->
                <sch:let 
                    name="threeVariantsInstruction"
                    value="'Provide 3 variants of summarizing the given DITA topic, that could be later used as short descriptions for this topic.
                    Respond with just the text content, no XML.'"/>
                <sch:let name="topicContent" value="."/>
                <sch:let name="theThreeVariants" value="ai:transform-content($threeVariantsInstruction, $topicContent)"/>
                
                <!-- Step 2: get the advantages and disadvantages of each variant -->
                <sch:let name="advAndDisadvInstr" value="'For each of the given text variants,
                    analyze how good a short description it is by enumerating 3 strong points and 3 weak points.'"/>
                <sch:let name="assistantAdvAndDisadv" value="ai:transform-content($advAndDisadvInstr, $theThreeVariants)"/>
                
                <!-- Step 3: ask the AI to choose the best variant -->
                <sch:let name="assistantBestVariant" value="ai:transform-content(
                    'You will act as a technical writer. You are tasked to choose the best short description for a topic.
                    You will take into account 3 variants and their strong points and weak points.
                    You should answer with the text of the best short description among the 3 variants.
                    Do not generate another variant, but choose one of the 3.',
                    
                    $threeVariantsInstruction, $theThreeVariants,
                    
                    $advAndDisadvInstr, $assistantAdvAndDisadv,
                    
                    $topicContent)"/>
                
                <!-- Perform the replacement fix using the best variant -->
                <sqf:add match="body" position="before" node-type="element" target="shortdesc" select="$assistantBestVariant"/>
            </sqf:fix>
        </sch:rule>
    </sch:pattern>
    
    
    <!-- Missing table cells validation -->
    <sch:pattern>
        <sch:rule context="table[@id='success_criteria_level_1']">
            <sch:let name="minColumsNo" value="min(.//row/count(entry))"/>
            <sch:let name="reqColumsNo" value="max(.//row/count(entry))"/>
            <sch:assert test="$minColumsNo >= $reqColumsNo" id="tableCellsID">Some table cells are missing. The
                number of cells for each row must be <sch:value-of select="$reqColumsNo"/>. Add the necesary cells.</sch:assert>
        </sch:rule>
    </sch:pattern>
    
    
    <!-- Long short description -->
    <sch:pattern>
        <sch:rule context="shortdesc">
            <sch:report 
                test="count(tokenize(.,'\s+')) > 85"
                role="warn"
                sqf:fix="rephrase">This short description is not short anymore...
            </sch:report>
            <sqf:fix id="rephrase">
                <sqf:description><sqf:title>Make short description short again</sqf:title></sqf:description>
                
                <!-- Step 1: rephrase in 3 possible ways -->
                <sch:let 
                    name="initialInstr"
                    value="'Provide 3 variants of rephrasing the given text in strictly less than 75 words.'"/>
                <sch:let name="currentShortDesc" value="text()"/>
                <sch:let name="threeVariants" value="ai:transform-content($initialInstr, $currentShortDesc)"/>
                
                <!-- Step 2: get the advantages and disadvantages of each variant -->
                <sch:let name="advAndDisadvInstr" value="'For each of the given text variants,
                    enumerate 3 strong points and 3 weak points about how it is written.'"/>
                <sch:let name="assistantAdvAndDisadv" value="ai:transform-content($advAndDisadvInstr, $threeVariants)"/>
                
                <!-- Step 3: ask the AI to choose the best variant -->
                <sch:let 
                    name="bestVariantChoosingInstruction" 
                    value="'Based on the strong points and weak point of the way each text is written, choose the one of them that best summarizes
                    the following text and is the best written. Respond with just the best text, nothing else.'"/>
                <sch:let name="assistantBestVariant" value="ai:transform-content(
                    $bestVariantChoosingInstruction,
                    
                    $initialInstr, $threeVariants,
                    
                    $advAndDisadvInstr, $assistantAdvAndDisadv,
                    
                    $currentShortDesc)"/>
                
                <!-- Perform the replacement fix using the best variant -->
                <sqf:replace match="$currentShortDesc" select="$assistantBestVariant"/>
            </sqf:fix>
        </sch:rule>
    </sch:pattern>
    
    
</sch:schema>
